@@ -1,5 +1,15 @@
-from . import (cfg, template, parameters, resolve, actions, events,
-               outputs, dashboard, ssm, route53)
+from . import (
+    cfg,
+    template,
+    parameters,
+    resolve,
+    actions,
+    events,
+    outputs,
+    dashboard,
+    ssm,
+    route53,
+)
 from .aws import myboto3
 from .log import logger, get_msg_client
 from .tools import smodule_to_class
@@ -10,9 +20,9 @@ class ibox_stack(object):
     def __init__(self, name, base_data, region=None):
         # aws clients/resource
         self.boto3 = myboto3(self, region)
-        self.cloudformation = self.boto3.resource('cloudformation')
-        self.s3 = self.boto3.client('s3')
-        self.client = self.boto3.client('cloudformation')
+        self.cloudformation = self.boto3.resource("cloudformation")
+        self.s3 = self.boto3.client("s3")
+        self.client = self.boto3.client("cloudformation")
 
         # set property
         self.name = name
@@ -69,6 +79,14 @@ class ibox_stack(object):
             self.stack.reload()
             return self.stack.stack_status
 
+    def rollback(self):
+        self.stack = self.cloudformation.Stack(self.name)
+        result = actions.rollback(self)
+
+        if result:
+            self.stack.reload()
+            return self.stack.stack_status
+
     def parameters(self, check=None):
         self.exports = self.cfg.exports
         self.template = template.get_template(self)
@@ -77,12 +95,13 @@ class ibox_stack(object):
             parameters.add_stack_params_as_args(self, parser)
             return self.stack_parsed_args, self.parameters
         else:
-            logger.info(f'{self.name} Parameters:')
+            logger.info(f"{self.name} Parameters:")
             parser.print_help()
 
     def info(self):
         self.stack = self.cloudformation.Stack(self.name)
-        outputs.show(self, 'before')
+        self.template = template.get_template(self)
+        outputs.show(self, "before")
         parameters.show_override(self)
 
     def log(self):
@@ -96,7 +115,7 @@ class ibox_stack(object):
         resolve.show(self)
 
     def ssm(self):
-        self.ssm = self.boto3.client('ssm')
+        self.ssm = self.boto3.client("ssm")
         ssm.put_parameters(self, self.bdata)
 
     def replicate(self, ssm_map, iregion):
@@ -104,16 +123,16 @@ class ibox_stack(object):
         self.cfg.stacks = iregion.cfg.stacks
         # pprint(ssm_map)
         for n, v in ssm_map.items():
-            if n.startswith(f'{self.name}/'):
-                parameter = n.split('/')[1]
+            if n.startswith(f"{self.name}/"):
+                parameter = n.split("/")[1]
                 setattr(self.cfg, parameter, v)
 
-        result = getattr(self, f'{self.cfg.command_replicate}')()
+        result = getattr(self, f"{self.cfg.command_replicate}")()
 
         return result
 
     def mylog(self, msg, chat=True):
-        message = f'{self.name} # {msg}'
+        message = f"{self.name} # {msg}"
         try:
             print(message)
         except IOError:
@@ -123,21 +142,55 @@ class ibox_stack(object):
         if client and chat:
             try:
                 client.chat_postMessage(
-                    channel=f'#{cfg.slack_channel}',
+                    channel=f"#{cfg.slack_channel}",
                     text=message,
-                    username=os.environ['IBOX_SLACK_USER'],
-                    icon_emoji=':robot_face:')
+                    username=os.environ["IBOX_SLACK_USER"],
+                    icon_emoji=":robot_face:",
+                )
             except Exception as e:
-                logger.warning(f'Error sending message to channel: {e}')
+                logger.warning(f"Error sending message to channel: {e}")
 
     def dash(self):
         dashboard.add_stack(self)
 
     def r53(self):
-        self.route53 = self.boto3.client('route53')
+        self.route53 = self.boto3.client("route53")
         result = route53.create(self)
         return result
 
+    def stackset_update(self):
+        cfg.fields = cfg.STACKSET_INSTANCES_SHOW_TABLE_FIELDS
+        self.exports = self.cfg.exports
+        self.template = template.get_template(self, stackset=True)
+        self.stack = True
+        parameters.process(self)
+        result = actions.stackset_update(self)
+        if result:
+            return {self.name: None}
+
+    def stackset_info(self):
+        self.exports = self.cfg.exports
+        self.template = template.get_template(self, stackset=True)
+        self.stack = True
+        if not self.cfg.compact:
+            self.cfg.OUT_WIDTH = 80
+        parameters.show_override(self)
+
+    def stackset_parameters(self):
+        self.exports = self.cfg.exports
+        self.template = template.get_template(self, stackset=True)
+        parser = parameters.get_stack_parameter_parser(self)
+        logger.info(f"{self.name} Parameters:")
+        parser.print_help()
+
+    def stackset_show(self):
+        result = actions.stackset_show(self)
+
+    def stackset_instances(self):
+        cfg.fields = cfg.STACKSET_INSTANCES_SHOW_TABLE_FIELDS
+        cfg.output = "text"
+        result = actions.stackset_instances(self)
+
 
 def exec_command(name, data, command, region=None, **kwargs):
     istack = ibox_stack(name, data, region)
